from fingest.plugin import data_fixture
from fingest.types import BaseFixture, JSONFixture


@data_fixture("test.json", description="JSON File Foo Bar")
class JsonData(JSONFixture): ...


@data_fixture("test.xml", description="XML File Foo Bar")
class XMLData(BaseFixture): ...


@data_fixture("test.csv", description="CSV File FOO Bar")
class CSV(BaseFixture):
    """CSV File"""

    ...


@data_fixture("test.json", description="Func Bases")
def json_test_file(data):
    """Json File in func"""
    return data



import pytest


@pytest.fixture
def fingest():
    return True


def test_fingest(fingest):
    assert fingest


def test_data_fixture(JsonData):
    assert JsonData.data.get("Foo") == "Bar"


def test_data_fixtute(JsonData):
    assert JsonData.length() == 1


def test_xml(XMLData):
    assert XMLData


def test_csv(CSV):
    assert len(CSV.data) == 5


def test_json_func(json_test_file):
    assert json_test_file.get("Foo") == "Bar"



import pytest
import json
import csv
from lxml import etree
from pathlib import Path

_fixture_registry = {}


def pytest_addoption(parser):
    parser.addini(
        name="fingest_fixture_path",
        help="Base path for fixture data files",
        default="data",
    )


def pytest_configure(config):
    data_path = config.getini("fingest_fixture_path")
    config.fingest_fixture_path = data_path


def data_fixture(file_path: str, description: str = ""):
    """
    Decorator: register class as a data-backed fixture with optional
    Description.
    """

    def wrapper(obj):
        _fixture_registry[obj.__name__] = {
            "obj": obj,
            "path": Path(file_path),
            "description": description,
            "is_class": isinstance(obj, type),
        }
        return obj

    return wrapper


def _load_data(path: Path):
    """Loads data from file.

    params:
    path: Path to the data file.
    raises: ValueError in case of invalid path file extension.
    returns:
    """
    if path.suffix[1:].lower() == "json":
        with open(path, "r", encoding="utf-8") as f:
            return json.load(f)
    if path.suffix[1:].lower() == "csv":
        with open(path, newline="", encoding="utf-8") as f:
            return list(csv.DictReader(f))
    if path.suffix[1:].lower() == "xml":
        return etree.parse(path)
    else:
        raise ValueError(f"Unsupported format: {path.suffix[1:].lower()}")


class FixtureWrapper:
    def __init__(self, instance, description: str):
        self._instance = instance
        self._description = description

    def __getattr__(self, item):
        return getattr(self._instance, item)

    def __repr__(self):
        return f"{self._description}"

    def __str__(self):
        return f"{self._instance}  (Fixture description: {self._description})"


def pytest_sessionstart(session):
    """
    Generate real fixtures at test session start.
    """
    data_root = getattr(session.config, "fingest_fixture_path", "data")
    for name, info in _fixture_registry.items():
        obj = info["obj"]
        path = Path(data_root) / Path(info["path"])
        desc = info["description"]
        is_class = info["is_class"]

        @pytest.fixture(name=name)
        def _fixture(obj=obj, path=path, desc=desc, is_class=is_class):
            data = _load_data(path)
            result = obj(data) if is_class else obj(data)
            return FixtureWrapper(result, desc)

        globals()[name] = _fixture



class BaseFixture(object):
    def __init__(self, data):
        self.data = data


class JSONFixture(BaseFixture):
    def keys(self):
        return self.data.keys()

    def length(self):
        return len(self.data)
